% Code_Figure6_10 - The code generates Figure 6.10 and Figure 6.11 of the book 
%                   'Data Science for Nano Image Analysis'
%                  written by Chiwoo Park and Yu Ding, 2021, Springer
%                  Nature 
%                
%               - implement  Section 4.2.2 of the book.
%
% Copyright ©2020 reserved to Chiwoo Park (cpark5@fsu.edu) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
clear all;
close all;

data_directory='./test_cases/mat_data/';
image_directory='./test_cases/image_data/';
result_directory='./test_cases/results/';

% create directories if they do not exist
mkdir(data_directory);
mkdir(image_directory);
mkdir(result_directory);

rep_num=50; % number of replicated experiments for the same experimental configuration.
V_num=[5,10,15,20,25];  % number of missing voids in lattice
Var_level=[0.05 0.15 0.25 0.35 0.45 0.55 0.65 0.75 0.85 0.95]; % image noise variance
mode={'uniform','1','1c','2','2c'}; % image pattern 
v_num_name={'void_5','void_10','void_15','void_20','void_25'}; 

% generate simulated datasets
generate_synthetic_cases(V_num, Var_level, v_num_name, mode, rep_num, data_directory, image_directory)

% run simulation
[FPS_dis, FNS_dis, EXP_config] = test_with_synthetic_cases(V_num, Var_level, v_num_name, mode, rep_num, data_directory, result_directory);

% summarize the results
N_Sim_Cases = size(FPS_dis, 1);
for n = 1:N_Sim_Cases
    
    no_void = V_num(EXP_config(n, 1));
    noise_var = Var_level(EXP_config(n, 2));
    mode_used  = mode{EXP_config(n, 3)};
    
    % number of false positives over "rep_num" number of replications
    average_FPs = mean(FPS_dis(n, :));
    std_FPs = std(FPS_dis(n, :));
    
    % number of false negatives over "rep_num" number of replications
    average_FNs = mean(FNS_dis(n, :));
    std_FNs = std(FNS_dis(n, :));
end